function OrderHallMissionsFrame_EnhancementUpdate(self)
	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		local missions;
		if (self.showInProgress) then
			missions = self.inProgressMissions;
		else
			missions = self.availableMissions;
		end
		local numMissions = #missions;
		local scrollFrame = self.listScroll;
		local offset = HybridScrollFrame_GetOffset(scrollFrame);
		local buttons = scrollFrame.buttons;
		local numButtons = #buttons;

		for i = 1, numButtons do
			local button = buttons[i];
			local index = offset + i;
			if (index <= numMissions) then
				local mission = missions[index];
				if (not mission.inProgress) then
					local duration = mission.duration;
					if (mission.durationSeconds >= GARRISON_LONG_MISSION_TIME) then
						duration = string.format(GARRISON_LONG_MISSION_TIME_FORMAT, duration);
					end
					local durationText = string.format(PARENS_TEMPLATE, duration);
					local durationTextLength = string.len(durationText);
					if (string.sub(durationText, durationTextLength) == ")") then
						local remainingSeconds = mission.offerEndTime - GetTime();
						local colorCode, colorCodeEnd = "", "";
						if (remainingSeconds < 8 * 3600) then
							colorCode, colorCodeEnd = RED_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
						elseif (remainingSeconds < 24 * 3600) then
							colorCode, colorCodeEnd = YELLOW_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
						end
						button.Summary:SetText(string.sub(durationText, 2, durationTextLength - 1)..", "..colorCode..string.format(SFE_EXPIRES_IN, mission.offerTimeRemaining)..colorCodeEnd);
					end
				end

				button.Title:SetPoint("LEFT", 165, 10);
				button.Title:SetWidth(655 - #mission.rewards * 65);
				button.Summary:ClearAllPoints();
				button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);
			end
		end
	end
end

local SfeOrderHallController = CreateFrame("Frame");
SfeOrderHallController:RegisterEvent("ADDON_LOADED");
SfeOrderHallController:SetScript("OnEvent", function(self, event, name)
		if ((event == "ADDON_LOADED") and (name == "Blizzard_OrderHallUI")) then
			if (SFE_ORDERHALL_MISSIONS_ENHANCED == nil) then
				SFE_ORDERHALL_MISSIONS_ENHANCED = true;
			end

			if (OrderHallMissionsEnhancedCheckbox == nil) then
				OrderHallMissionsEnhancedCheckbox = CreateFrame("CheckButton", "OrderHallMissionsEnhancedCheckbox", OrderHallMissionFrame, "OptionsBaseCheckButtonTemplate");
				OrderHallMissionFrame.EnhancedCheckbox = OrderHallMissionsEnhancedCheckbox;
				OrderHallMissionsEnhancedCheckbox:SetHitRectInsets(0, 0, 0, 0);

				OrderHallMissionsEnhancedCheckbox:SetPoint("RIGHT", OrderHallMissionFrame.CloseButton, "LEFT", 7, 0);
				OrderHallMissionsEnhancedCheckbox:SetAlpha(0.1);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnClick", function(self, button, down)
						SFE_ORDERHALL_MISSIONS_ENHANCED = self:GetChecked();
						OrderHallMissionFrame.MissionTab.MissionList:Update(OrderHallMissionFrame.MissionTab.MissionList);
						if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
							PlaySound("igMainMenuOptionCheckBoxOn");
						else
							PlaySound("igMainMenuOptionCheckBoxOff");
						end
					end);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnEnter", function(self, motion)
						self:SetAlpha(1.0);
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
						GameTooltip:SetText(SFE_ENHANCED_UI, nil, nil, nil, nil, true);
					end);
				OrderHallMissionsEnhancedCheckbox:SetScript("OnLeave", function(self, motion)
						self:SetAlpha(0.1);
						GameTooltip_Hide(self, motion);
					end);
			end
			OrderHallMissionsEnhancedCheckbox:SetChecked(SFE_ORDERHALL_MISSIONS_ENHANCED);

			local oldOrderHallMissionFrame_MissionTab_MissionList_Update = OrderHallMissionFrame.MissionTab.MissionList.Update;
			function OrderHallMissionFrame.MissionTab.MissionList:Update()
				oldOrderHallMissionFrame_MissionTab_MissionList_Update(self);
				OrderHallMissionsFrame_EnhancementUpdate(self);
			end
		end
	end);
